@@ -16,13 +16,10 @@
 # under the License.
 from __future__ import annotations
 
-import inspect
-from textwrap import dedent
-from typing import Callable, Sequence
+from typing import Callable
 
 from airflow.decorators.base import DecoratedOperator, TaskDecorator, task_decorator_factory
 from airflow.operators.python import ExternalPythonOperator
-from airflow.utils.decorators import remove_task_decorator
 
 
 class _PythonExternalDecoratedOperator(DecoratedOperator, ExternalPythonOperator):
@@ -41,14 +38,7 @@ class _PythonExternalDecoratedOperator(DecoratedOperator, ExternalPythonOperator
         multiple XCom values. Dict will unroll to XCom values with its keys as XCom keys. Defaults to False.
     """
 
-    template_fields: Sequence[str] = ('op_args', 'op_kwargs')
-    template_fields_renderers = {"op_args": "py", "op_kwargs": "py"}
-
-    # since we won't mutate the arguments, we should just do the shallow copy
-    # there are some cases we can't deepcopy the objects (e.g protobuf).
-    shallow_copy_attrs: Sequence[str] = ('python_callable',)
-
-    custom_operator_name: str = '@task.external_python'
+    custom_operator_name: str = "@task.external_python"
 
     def __init__(self, *, python_callable, op_args, op_kwargs, **kwargs) -> None:
         kwargs_to_upstream = {
@@ -64,12 +54,6 @@ class _PythonExternalDecoratedOperator(DecoratedOperator, ExternalPythonOperator
             **kwargs,
         )
 
-    def get_python_source(self):
-        raw_source = inspect.getsource(self.python_callable)
-        res = dedent(raw_source)
-        res = remove_task_decorator(res, "@task.external_python")
-        return res
-
 
 def external_python_task(
     python: str | None = None,
