@@ -15,15 +15,22 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-from typing import Dict, List, Optional, Set, Union
+from __future__ import annotations
 
+from typing import TYPE_CHECKING, Sequence
+
+from airflow.callbacks.base_callback_sink import BaseCallbackSink
+from airflow.callbacks.callback_requests import CallbackRequest
 from airflow.configuration import conf
-from airflow.executors.base_executor import CommandType, EventBufferValueType, QueuedTaskInstanceType
 from airflow.executors.celery_executor import CeleryExecutor
 from airflow.executors.kubernetes_executor import KubernetesExecutor
-from airflow.models.taskinstance import SimpleTaskInstance, TaskInstance, TaskInstanceKey
 from airflow.utils.log.logging_mixin import LoggingMixin
 
+if TYPE_CHECKING:
+    from airflow.executors.base_executor import CommandType, EventBufferValueType, QueuedTaskInstanceType
+    from airflow.models.taskinstance import SimpleTaskInstance, TaskInstance
+    from airflow.models.taskinstancekey import TaskInstanceKey
+
 
 class CeleryKubernetesExecutor(LoggingMixin):
     """
@@ -34,50 +41,66 @@ class CeleryKubernetesExecutor(LoggingMixin):
     otherwise, CeleryExecutor is used.
     """
 
-    KUBERNETES_QUEUE = conf.get('celery_kubernetes_executor', 'kubernetes_queue')
+    supports_ad_hoc_ti_run: bool = True
+    supports_pickling: bool = True
+    supports_sentry: bool = False
 
-    def __init__(self, celery_executor, kubernetes_executor):
+    is_local: bool = False
+    is_single_threaded: bool = False
+    is_production: bool = True
+
+    serve_logs: bool = False
+    change_sensor_mode_to_reschedule: bool = False
+
+    callback_sink: BaseCallbackSink | None = None
+
+    KUBERNETES_QUEUE = conf.get("celery_kubernetes_executor", "kubernetes_queue")
+
+    def __init__(self, celery_executor: CeleryExecutor, kubernetes_executor: KubernetesExecutor):
         super().__init__()
-        self._job_id: Optional[str] = None
+        self._job_id: int | None = None
         self.celery_executor = celery_executor
         self.kubernetes_executor = kubernetes_executor
+        self.kubernetes_executor.kubernetes_queue = self.KUBERNETES_QUEUE
 
     @property
-    def queued_tasks(self) -> Dict[TaskInstanceKey, QueuedTaskInstanceType]:
-        """Return queued tasks from celery and kubernetes executor"""
+    def queued_tasks(self) -> dict[TaskInstanceKey, QueuedTaskInstanceType]:
+        """Return queued tasks from celery and kubernetes executor."""
         queued_tasks = self.celery_executor.queued_tasks.copy()
         queued_tasks.update(self.kubernetes_executor.queued_tasks)
 
         return queued_tasks
 
     @property
-    def running(self) -> Set[TaskInstanceKey]:
-        """Return running tasks from celery and kubernetes executor"""
+    def running(self) -> set[TaskInstanceKey]:
+        """Return running tasks from celery and kubernetes executor."""
         return self.celery_executor.running.union(self.kubernetes_executor.running)
 
     @property
-    def job_id(self):
+    def job_id(self) -> int | None:
         """
-        This is a class attribute in BaseExecutor but since this is not really an executor, but a wrapper
-        of executors we implement as property so we can have custom setter.
+        Inherited attribute from BaseExecutor.
+
+        Since this is not really an executor, but a wrapper of executors
+        we implemented it as property, so we can have custom setter.
         """
         return self._job_id
 
     @job_id.setter
-    def job_id(self, value):
-        """job_id is manipulated by SchedulerJob.  We must propagate the job_id to wrapped executors."""
+    def job_id(self, value: int | None) -> None:
+        """Expose job ID for SchedulerJob."""
         self._job_id = value
         self.kubernetes_executor.job_id = value
         self.celery_executor.job_id = value
 
     def start(self) -> None:
-        """Start celery and kubernetes executor"""
+        """Start celery and kubernetes executor."""
         self.celery_executor.start()
         self.kubernetes_executor.start()
 
     @property
-    def slots_available(self):
-        """Number of new tasks this executor instance can accept"""
+    def slots_available(self) -> int:
+        """Number of new tasks this executor instance can accept."""
         return self.celery_executor.slots_available
 
     def queue_command(
@@ -85,9 +108,9 @@ class CeleryKubernetesExecutor(LoggingMixin):
         task_instance: TaskInstance,
         command: CommandType,
         priority: int = 1,
-        queue: Optional[str] = None,
-    ):
-        """Queues command via celery or kubernetes executor"""
+        queue: str | None = None,
+    ) -> None:
+        """Queues command via celery or kubernetes executor."""
         executor = self._router(task_instance)
         self.log.debug("Using executor: %s for %s", executor.__class__.__name__, task_instance.key)
         executor.queue_command(task_instance, command, priority, queue)
@@ -96,31 +119,41 @@ class CeleryKubernetesExecutor(LoggingMixin):
         self,
         task_instance: TaskInstance,
         mark_success: bool = False,
-        pickle_id: Optional[str] = None,
+        pickle_id: int | None = None,
         ignore_all_deps: bool = False,
         ignore_depends_on_past: bool = False,
+        wait_for_past_depends_before_skipping: bool = False,
         ignore_task_deps: bool = False,
         ignore_ti_state: bool = False,
-        pool: Optional[str] = None,
-        cfg_path: Optional[str] = None,
+        pool: str | None = None,
+        cfg_path: str | None = None,
     ) -> None:
-        """Queues task instance via celery or kubernetes executor"""
-        executor = self._router(SimpleTaskInstance(task_instance))
+        """Queues task instance via celery or kubernetes executor."""
+        from airflow.models.taskinstance import SimpleTaskInstance
+
+        executor = self._router(SimpleTaskInstance.from_ti(task_instance))
         self.log.debug(
             "Using executor: %s to queue_task_instance for %s", executor.__class__.__name__, task_instance.key
         )
         executor.queue_task_instance(
-            task_instance,
-            mark_success,
-            pickle_id,
-            ignore_all_deps,
-            ignore_depends_on_past,
-            ignore_task_deps,
-            ignore_ti_state,
-            pool,
-            cfg_path,
+            task_instance=task_instance,
+            mark_success=mark_success,
+            pickle_id=pickle_id,
+            ignore_all_deps=ignore_all_deps,
+            ignore_depends_on_past=ignore_depends_on_past,
+            wait_for_past_depends_before_skipping=wait_for_past_depends_before_skipping,
+            ignore_task_deps=ignore_task_deps,
+            ignore_ti_state=ignore_ti_state,
+            pool=pool,
+            cfg_path=cfg_path,
         )
 
+    def get_task_log(self, ti: TaskInstance, try_number: int) -> tuple[list[str], list[str]]:
+        """Fetch task log from Kubernetes executor"""
+        if ti.queue == self.kubernetes_executor.kubernetes_queue:
+            return self.kubernetes_executor.get_task_log(ti=ti, try_number=try_number)
+        return [], []
+
     def has_task(self, task_instance: TaskInstance) -> bool:
         """
         Checks if a task is either queued or running in either celery or kubernetes executor.
@@ -133,15 +166,17 @@ class CeleryKubernetesExecutor(LoggingMixin):
         )
 
     def heartbeat(self) -> None:
-        """Heartbeat sent to trigger new jobs in celery and kubernetes executor"""
+        """Heartbeat sent to trigger new jobs in celery and kubernetes executor."""
         self.celery_executor.heartbeat()
         self.kubernetes_executor.heartbeat()
 
-    def get_event_buffer(self, dag_ids=None) -> Dict[TaskInstanceKey, EventBufferValueType]:
+    def get_event_buffer(
+        self, dag_ids: list[str] | None = None
+    ) -> dict[TaskInstanceKey, EventBufferValueType]:
         """
-        Returns and flush the event buffer from celery and kubernetes executor
+        Return and flush the event buffer from celery and kubernetes executor.
 
-        :param dag_ids: to dag_ids to return events for, if None returns all
+        :param dag_ids: dag_ids to return events for, if None returns all
         :return: a dict of events
         """
         cleared_events_from_celery = self.celery_executor.get_event_buffer(dag_ids)
@@ -149,7 +184,7 @@ class CeleryKubernetesExecutor(LoggingMixin):
 
         return {**cleared_events_from_celery, **cleared_events_from_kubernetes}
 
-    def try_adopt_task_instances(self, tis: List[TaskInstance]) -> List[TaskInstance]:
+    def try_adopt_task_instances(self, tis: Sequence[TaskInstance]) -> Sequence[TaskInstance]:
         """
         Try to adopt running task instances that have been abandoned by a SchedulerJob dying.
 
@@ -157,45 +192,55 @@ class CeleryKubernetesExecutor(LoggingMixin):
         re-scheduling)
 
         :return: any TaskInstances that were unable to be adopted
-        :rtype: list[airflow.models.TaskInstance]
         """
-        celery_tis = []
-        kubernetes_tis = []
-        abandoned_tis = []
-        for ti in tis:
-            if ti.queue == self.KUBERNETES_QUEUE:
-                kubernetes_tis.append(ti)
-            else:
-                celery_tis.append(ti)
-        abandoned_tis.extend(self.celery_executor.try_adopt_task_instances(celery_tis))
-        abandoned_tis.extend(self.kubernetes_executor.try_adopt_task_instances(kubernetes_tis))
-        return abandoned_tis
+        celery_tis = [ti for ti in tis if ti.queue != self.KUBERNETES_QUEUE]
+        kubernetes_tis = [ti for ti in tis if ti.queue == self.KUBERNETES_QUEUE]
+        return [
+            *self.celery_executor.try_adopt_task_instances(celery_tis),
+            *self.kubernetes_executor.try_adopt_task_instances(kubernetes_tis),
+        ]
+
+    def cleanup_stuck_queued_tasks(self, tis: list[TaskInstance]) -> list[str]:
+        celery_tis = [ti for ti in tis if ti.queue != self.KUBERNETES_QUEUE]
+        kubernetes_tis = [ti for ti in tis if ti.queue == self.KUBERNETES_QUEUE]
+        return [
+            *self.celery_executor.cleanup_stuck_queued_tasks(celery_tis),
+            *self.kubernetes_executor.cleanup_stuck_queued_tasks(kubernetes_tis),
+        ]
 
     def end(self) -> None:
-        """End celery and kubernetes executor"""
+        """End celery and kubernetes executor."""
         self.celery_executor.end()
         self.kubernetes_executor.end()
 
     def terminate(self) -> None:
-        """Terminate celery and kubernetes executor"""
+        """Terminate celery and kubernetes executor."""
         self.celery_executor.terminate()
         self.kubernetes_executor.terminate()
 
-    def _router(self, simple_task_instance: SimpleTaskInstance) -> Union[CeleryExecutor, KubernetesExecutor]:
+    def _router(self, simple_task_instance: SimpleTaskInstance) -> CeleryExecutor | KubernetesExecutor:
         """
-        Return either celery_executor or kubernetes_executor
+        Return either celery_executor or kubernetes_executor.
 
         :param simple_task_instance: SimpleTaskInstance
         :return: celery_executor or kubernetes_executor
-        :rtype: Union[CeleryExecutor, KubernetesExecutor]
         """
         if simple_task_instance.queue == self.KUBERNETES_QUEUE:
             return self.kubernetes_executor
         return self.celery_executor
 
-    def debug_dump(self):
-        """Called in response to SIGUSR2 by the scheduler"""
+    def debug_dump(self) -> None:
+        """Called in response to SIGUSR2 by the scheduler."""
         self.log.info("Dumping CeleryExecutor state")
         self.celery_executor.debug_dump()
         self.log.info("Dumping KubernetesExecutor state")
         self.kubernetes_executor.debug_dump()
+
+    def send_callback(self, request: CallbackRequest) -> None:
+        """Sends callback for execution.
+
+        :param request: Callback request to be executed.
+        """
+        if not self.callback_sink:
+            raise ValueError("Callback sink is not ready.")
+        self.callback_sink.send(request)
