@@ -14,25 +14,20 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-"""
-Classes for interacting with Kubernetes API
-"""
+"""Classes for interacting with Kubernetes API."""
+from __future__ import annotations
 
-import abc
-import sys
+from abc import ABC, abstractmethod
 from functools import reduce
 
-if sys.version_info >= (3, 4):
-    ABC = abc.ABC
-else:
-    ABC = abc.ABCMeta('ABC', (), {})
+from kubernetes.client import models as k8s
 
 
 class K8SModel(ABC):
-
     """
-    These Airflow Kubernetes models are here for backwards compatibility
-    reasons only. Ideally clients should use the kubernetes api
+    Airflow Kubernetes models are here for backwards compatibility reasons only.
+
+    Ideally clients should use the kubernetes API
     and the process of
 
         client input -> Airflow k8s models -> k8s models
@@ -41,38 +36,24 @@ class K8SModel(ABC):
     `attach_to_pod` method so that they integrate with the kubernetes client.
     """
 
-    @abc.abstractmethod
-    def attach_to_pod(self, pod):
+    @abstractmethod
+    def attach_to_pod(self, pod: k8s.V1Pod) -> k8s.V1Pod:
         """
+        Attaches to pod.
+
         :param pod: A pod to attach this Kubernetes object to
-        :type pod: kubernetes.client.models.V1Pod
         :return: The pod with the object attached
         """
 
-    def as_dict(self):
-        res = {}
-        if hasattr(self, "__slots__"):
-            for s in self.__slots__:
-                if hasattr(self, s):
-                    res[s] = getattr(self, s)
-        if hasattr(self, "__dict__"):
-            res_dict = self.__dict__.copy()
-            res_dict.update(res)
-            return res_dict
-        return res
 
-
-def append_to_pod(pod, k8s_objects):
+def append_to_pod(pod: k8s.V1Pod, k8s_objects: list[K8SModel] | None):
     """
-    Attach Kubernetes objects to the given POD
+    Attach additional specs to an existing pod object.
 
     :param pod: A pod to attach a list of Kubernetes objects to
-    :type pod: kubernetes.client.models.V1Pod
     :param k8s_objects: a potential None list of K8SModels
-    :type k8s_objects: Optional[List[K8SModel]]
     :return: pod with the objects attached if they exist
     """
     if not k8s_objects:
         return pod
-    new_pod = reduce(lambda p, o: o.attach_to_pod(p), k8s_objects, pod)
-    return new_pod
+    return reduce(lambda p, o: o.attach_to_pod(p), k8s_objects, pod)
