@@ -32,12 +32,14 @@ from datetime import datetime
 from pathlib import Path
 from typing import TYPE_CHECKING, Callable, TypeVar, cast
 
+from sqlalchemy.orm import Session
+
 from airflow import settings
-from airflow.exceptions import AirflowException
+from airflow.exceptions import AirflowException, RemovedInAirflow3Warning
 from airflow.utils import cli_action_loggers
 from airflow.utils.log.non_caching_file_handler import NonCachingFileHandler
 from airflow.utils.platform import getuser, is_terminal_support_colors
-from airflow.utils.session import provide_session
+from airflow.utils.session import NEW_SESSION, provide_session
 
 T = TypeVar("T", bound=Callable)
 
@@ -101,9 +103,11 @@ def action_cli(func=None, check_db=True):
             try:
                 # Check and run migrations if necessary
                 if check_db:
+                    from airflow.configuration import conf
                     from airflow.utils.db import check_and_run_migrations, synchronize_log_template
 
-                    check_and_run_migrations()
+                    if conf.getboolean("database", "check_migrations"):
+                        check_and_run_migrations()
                     synchronize_log_template()
                 return f(*args, **kwargs)
             except Exception as e:
@@ -251,7 +255,7 @@ def get_dags(subdir: str | None, dag_id: str, use_regex: bool = False):
 
 
 @provide_session
-def get_dag_by_pickle(pickle_id, session=None):
+def get_dag_by_pickle(pickle_id: int, session: Session = NEW_SESSION) -> DAG:
     """Fetch DAG from the database using pickling"""
     from airflow.models import DagPickle
 
@@ -333,6 +337,18 @@ def should_use_colors(args) -> bool:
     return is_terminal_support_colors()
 
 
+def should_ignore_depends_on_past(args) -> bool:
+    if args.ignore_depends_on_past:
+        warnings.warn(
+            "Using `--ignore-depends-on-past` is Deprecated."
+            "Please use `--depends-on-past ignore` instead.",
+            RemovedInAirflow3Warning,
+            stacklevel=2,
+        )
+        return True
+    return args.depends_on_past == "ignore"
+
+
 def suppress_logs_and_warning(f: T) -> T:
     """
     Decorator to suppress logging and warning messages
