@@ -14,14 +14,20 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-
 """Platform and system specific function."""
+from __future__ import annotations
+
 import getpass
 import logging
 import os
 import pkgutil
+import platform
 import sys
 
+from airflow.compat.functools import cache
+
+IS_WINDOWS = platform.system() == "Windows"
+
 log = logging.getLogger(__name__)
 
 
@@ -53,13 +59,14 @@ def get_airflow_git_version():
     """Returns the git commit hash representing the current version of the application."""
     git_version = None
     try:
-        git_version = str(pkgutil.get_data('airflow', 'git_version'), encoding="UTF-8")
+        git_version = str(pkgutil.get_data("airflow", "git_version"), encoding="UTF-8")
     except Exception as e:
         log.debug(e)
 
     return git_version
 
 
+@cache
 def getuser() -> str:
     """
     Gets the username associated with the current user, or error with a nice
