@@ -24,10 +24,8 @@ import pendulum
 from dateutil.relativedelta import relativedelta
 from pendulum.datetime import DateTime
 
-from airflow.settings import TIMEZONE
-
 # UTC time zone as a tzinfo instance.
-utc = pendulum.tz.timezone('UTC')
+utc = pendulum.tz.timezone("UTC")
 
 
 def is_localized(value):
@@ -104,6 +102,8 @@ def convert_to_utc(value: dt.datetime | None) -> DateTime | None:
         return value
 
     if not is_localized(value):
+        from airflow.settings import TIMEZONE
+
         value = pendulum.instance(value, TIMEZONE)
 
     return pendulum.instance(value.astimezone(utc))
@@ -133,6 +133,8 @@ def make_aware(value: dt.datetime | None, timezone: dt.tzinfo | None = None) ->
     :return: localized datetime in settings.TIMEZONE or timezone
     """
     if timezone is None:
+        from airflow.settings import TIMEZONE
+
         timezone = TIMEZONE
 
     if not value:
@@ -141,17 +143,17 @@ def make_aware(value: dt.datetime | None, timezone: dt.tzinfo | None = None) ->
     # Check that we won't overwrite the timezone of an aware datetime.
     if is_localized(value):
         raise ValueError(f"make_aware expects a naive datetime, got {value}")
-    if hasattr(value, 'fold'):
+    if hasattr(value, "fold"):
         # In case of python 3.6 we want to do the same that pendulum does for python3.5
         # i.e in case we move clock back we want to schedule the run at the time of the second
         # instance of the same clock time rather than the first one.
         # Fold parameter has no impact in other cases so we can safely set it to 1 here
         value = value.replace(fold=1)
-    localized = getattr(timezone, 'localize', None)
+    localized = getattr(timezone, "localize", None)
     if localized is not None:
         # This method is available for pytz time zones
         return localized(value)
-    convert = getattr(timezone, 'convert', None)
+    convert = getattr(timezone, "convert", None)
     if convert is not None:
         # For pendulum
         return convert(value)
@@ -168,6 +170,8 @@ def make_naive(value, timezone=None):
     :return: naive datetime
     """
     if timezone is None:
+        from airflow.settings import TIMEZONE
+
         timezone = TIMEZONE
 
     # Emulate the behavior of astimezone() on Python < 3.6.
@@ -190,8 +194,10 @@ def datetime(*args, **kwargs):
 
     :return: datetime.datetime
     """
-    if 'tzinfo' not in kwargs:
-        kwargs['tzinfo'] = TIMEZONE
+    if "tzinfo" not in kwargs:
+        from airflow.settings import TIMEZONE
+
+        kwargs["tzinfo"] = TIMEZONE
 
     return dt.datetime(*args, **kwargs)
 
@@ -203,6 +209,8 @@ def parse(string: str, timezone=None) -> DateTime:
     :param string: time string
     :param timezone: the timezone
     """
+    from airflow.settings import TIMEZONE
+
     return pendulum.parse(string, tz=timezone or TIMEZONE, strict=False)  # type: ignore
 
 
@@ -261,7 +269,7 @@ def td_format(td_object: None | dt.timedelta | float | int) -> str | None:
             return ""
         # distinguish between month/minute following strftime format
         # and take first char of each unit, i.e. years='y', days='d'
-        if key == 'minutes':
+        if key == "minutes":
             key = key.upper()
         key = key[0]
         return f"{value}{key}"
