@@ -38,8 +38,7 @@ from sqlalchemy import (
     event,
     func,
 )
-from sqlalchemy.ext.declarative import declared_attr
-from sqlalchemy.orm import backref, relationship
+from sqlalchemy.orm import backref, declared_attr, relationship
 
 from airflow.models.base import Base
 
@@ -145,14 +144,14 @@ class User(Model):
 
     __tablename__ = "ab_user"
     id = Column(Integer, primary_key=True)
-    first_name = Column(String(64), nullable=False)
-    last_name = Column(String(64), nullable=False)
+    first_name = Column(String(256), nullable=False)
+    last_name = Column(String(256), nullable=False)
     username = Column(
-        String(256).with_variant(String(256, collation="NOCASE"), "sqlite"), unique=True, nullable=False
+        String(512).with_variant(String(512, collation="NOCASE"), "sqlite"), unique=True, nullable=False
     )
     password = Column(String(256))
     active = Column(Boolean)
-    email = Column(String(256), unique=True, nullable=False)
+    email = Column(String(512), unique=True, nullable=False)
     last_login = Column(DateTime)
     login_count = Column(Integer)
     fail_login_count = Column(Integer)
@@ -240,13 +239,13 @@ class RegisterUser(Model):
 
     __tablename__ = "ab_register_user"
     id = Column(Integer, primary_key=True)
-    first_name = Column(String(64), nullable=False)
-    last_name = Column(String(64), nullable=False)
+    first_name = Column(String(256), nullable=False)
+    last_name = Column(String(256), nullable=False)
     username = Column(
-        String(256).with_variant(String(256, collation="NOCASE"), "sqlite"), unique=True, nullable=False
+        String(512).with_variant(String(512, collation="NOCASE"), "sqlite"), unique=True, nullable=False
     )
     password = Column(String(256))
-    email = Column(String(256), nullable=False)
+    email = Column(String(512), nullable=False)
     registration_date = Column(DateTime, default=datetime.datetime.now, nullable=True)
     registration_hash = Column(String(256))
 
