@@ -27,7 +27,7 @@ from flask import request, url_for
 from flask.helpers import flash
 from flask_appbuilder.forms import FieldConverter
 from flask_appbuilder.models.filters import BaseFilter
-from flask_appbuilder.models.sqla import filters as fab_sqlafilters
+from flask_appbuilder.models.sqla import Model, filters as fab_sqlafilters
 from flask_appbuilder.models.sqla.filters import get_field_setup_query, set_value_to_type
 from flask_appbuilder.models.sqla.interface import SQLAInterface
 from flask_babel import lazy_gettext
@@ -53,6 +53,7 @@ from airflow.www.widgets import AirflowDateTimePickerWidget
 
 if TYPE_CHECKING:
     from sqlalchemy.orm.query import Query
+    from sqlalchemy.orm.session import Session
     from sqlalchemy.sql.operators import ColumnOperators
 
     from airflow.www.fab_security.sqla.manager import SecurityManager
@@ -88,7 +89,7 @@ def get_try_count(try_number: int, state: State):
     return try_number + 1 if state in [State.DEFERRED, State.UP_FOR_RESCHEDULE] else try_number
 
 
-priority = [
+priority: list[None | TaskInstanceState] = [
     TaskInstanceState.FAILED,
     TaskInstanceState.UPSTREAM_FAILED,
     TaskInstanceState.UP_FOR_RETRY,
@@ -133,17 +134,28 @@ def get_mapped_summary(parent_instance, task_instances):
     }
 
 
-def encode_dag_run(dag_run: DagRun | None) -> dict[str, Any] | None:
+def get_dag_run_conf(
+    dag_run_conf: Any, *, json_encoder: type[json.JSONEncoder] = json.JSONEncoder
+) -> tuple[str | None, bool]:
+    conf: str | None = None
+
+    conf_is_json: bool = False
+    if isinstance(dag_run_conf, str):
+        conf = dag_run_conf
+    elif isinstance(dag_run_conf, (dict, list)) and any(dag_run_conf):
+        conf = json.dumps(dag_run_conf, sort_keys=True, cls=json_encoder, ensure_ascii=False)
+        conf_is_json = True
+
+    return conf, conf_is_json
+
+
+def encode_dag_run(
+    dag_run: DagRun | None, *, json_encoder: type[json.JSONEncoder] = json.JSONEncoder
+) -> dict[str, Any] | None:
     if not dag_run:
         return None
 
-    conf: str | None = None
-    conf_is_json: bool = False
-    if isinstance(dag_run.conf, str):
-        conf = dag_run.conf
-    elif isinstance(dag_run.conf, (dict, list)) and any(dag_run.conf):
-        conf = json.dumps(dag_run.conf, sort_keys=True)
-        conf_is_json = True
+    conf, conf_is_json = get_dag_run_conf(dag_run.conf, json_encoder=json_encoder)
 
     return {
         "run_id": dag_run.run_id,
@@ -746,7 +758,7 @@ class CustomSQLAInterface(SQLAInterface):
 
     """
 
-    def __init__(self, obj, session=None):
+    def __init__(self, obj, session: Session | None = None):
         super().__init__(obj, session=session)
 
         def clean_column_names():
@@ -800,6 +812,25 @@ class CustomSQLAInterface(SQLAInterface):
     filter_converter_class = AirflowFilterConverter
 
 
+class DagRunCustomSQLAInterface(CustomSQLAInterface):
+    """Custom interface to allow faster deletion.
+
+    The ``delete`` and ``delete_all`` methods are overridden to speed up
+    deletion when a DAG run has a lot of related task instances. Relying on
+    SQLAlchemy's cascading deletion is comparatively slow in this situation.
+    """
+
+    def delete(self, item: Model, raise_exception: bool = False) -> bool:
+        self.session.query(TaskInstance).where(TaskInstance.run_id == item.run_id).delete()
+        return super().delete(item, raise_exception=raise_exception)
+
+    def delete_all(self, items: list[Model]) -> bool:
+        self.session.query(TaskInstance).where(
+            TaskInstance.run_id.in_(item.run_id for item in items)
+        ).delete()
+        return super().delete_all(items)
+
+
 # This class is used directly (i.e. we can't tell Fab to use a different
 # subclass) so we have no other option than to edit the conversion table in
 # place
